; 该部分完成对地址的分配
io8255_a	equ	230h            ; 8255控制器A的I/O地址
io8255_b	equ	231h            ; 8255控制器B的I/O地址
io8255_c	equ	232h            ; 8255控制器C的I/O地址
io8255_k	equ	233h            ; 8255控制器的控制字寄存器地址
io8253_0	equ	210h            ; 8253计数器0的I/O地址
io8253_1	equ	211h            ; 8253计数器1的I/O地址
io8253_k	equ	213h            ; 8253控制字寄存器地址
io8259_0	equ	200h            ; 8259中断控制器的主I/O地址
io8259_1	equ	201h            ; 8259中断控制器的从I/O地址
AD_START	equ   220H           ; AD转换开始信号的I/O地址
AD_OE	equ   221H              ; AD输出使能信号的I/O地址
AD_EOI	equ   222H             ; AD结束中断信号的I/O地址

; 该部分完成数据段定义
data		segment
	count1	dw	0                 ; 中断计数器，初始值为0
	portcl		db	24H,44h,81h,82h,0ffh  ; 红绿灯状态字定义
	input		db	?                 ; 读取电位器数据的变量
	buf		db	0                 ; 电机相序的缓冲区
data		ends
	
st_ssg		segment	stack
	db	256	dup(?)               ; 定义256字节的栈空间
st_ssg		ends

; 该部分完成代码段定义
code	segment
	assume	cs:code,ds:data,ss:st_ssg ; 假定段寄存器的值
start:
	cli                         ; 禁用中断
	mov	ax,data                ; 将数据段地址加载到AX
	mov	ds,ax                  ; 设置数据段寄存器DS

; --------------------------------8259初始化--------------------------------
	mov	dx,io8259_0           ; 加载主8259的I/O地址到DX
	mov	al,00010011B          ; ICW1: 配置8259
	out	dx,al                 ; 将ICW1输出到8259
	mov	dx,io8259_1           ; 加载从8259的I/O地址到DX
	mov	al,00001000B          ; ICW2: 设置从8259的基地址
	out	dx,al                 ; 将ICW2输出到8259
	mov	al,00000001B          ; ICW4: 启用级联
	out	dx,al                 ; 将ICW4输出到8259

; --------------------------------定义中断向量------------------------------
	push	ds                    ; 保存数据段寄存器
	mov	ax,0                  ; 将AX清零
	mov	ds,ax                  ; 将DS设为0
	lea	ax,cs:int_proc       ; AX指向中断处理程序的入口地址
	mov	si,08h                ; 设置MIR0类型码为08h
	add	si,si                 ; SI = SI * 2
	add	si,si                 ; SI = SI * 2
	mov	ds:[si],ax            ; 设置中断向量表的IP值
	push	cs                   ; 保存代码段寄存器
	pop	ax                    ; 将CS值放入AX
	mov	ds:[si+2],ax          ; 设置中断向量表的CS值
	pop	ds                    ; 恢复数据段寄存器
	in	al,dx                  ; 从8259读取数据
	and	al,11111110b          ; 清除8259的第0位
	out	dx,al                 ; 输出到8259

; -----8253分频器，CNT0为模式3，CLK0输入时钟1MHz，OUT0输出周期10ms-----
	mov	ax,0                  ; 清零AX
	mov	al,00110110b          ; 设置8253计数器的控制字
	mov	dx,io8253_k           ; 加载8253控制字寄存器地址到DX
	out	dx,al                 ; 输出控制字到8253
	mov	ax,10000              ; 设置计数值为10000
	mov	dx,io8253_0           ; 加载8253计数器0的地址到DX
	out	dx,al                 ; 输出低字节
	mov	al,ah                 ; 获取高字节
	out	dx,al                 ; 输出高字节

; ---------8255初始化，PA、PB为输出，上PC口为输入，下PC口为输出-----------
	mov	dx,io8255_k           ; 加载8255控制字寄存器地址到DX
	mov	al,10001000b          ; 设置8255的控制字
	out	dx,al                 ; 输出控制字到8255

	mov buf,33h                ; 初始化电机相序
	sti                         ; 使能中断

; ---------------------------AD0809数据转换---------------------------------
lop1:	mov	dx,AD_START         ; 加载AD转换开始信号地址到DX
	out	dx,al                 ; 发送开始信号
l1:	mov	dx,AD_EOI            ; 加载AD结束中断信号地址到DX
	in	al,dx                  ; 从AD读取数据
	test	al,01h               ; 检查转换完成标志
	jz	l1                     ; 若未完成，则循环等待
	mov	dx,AD_OE              ; 加载AD输出使能信号地址到DX
	in	al,dx                  ; 读取AD输出数据
	mov	input,al              ; 存储AD转换结果

; ------------------------------电机驱动------------------------------------
out1:	mov	al,buf              ; 加载当前电机相序
	mov	dx,io8255_c           ; 加载8255控制器C的地址到DX
	out	dx,al                 ; 输出电机控制信号
lop:	call	delay               ; 调用延时程序
	mov	dx,io8255_c           ; 读取当前电机状态
	in	al,dx                  ; 从8255读取电机状态
sam:	test	al,80h              ; 检查状态位
	jnz	nx0                   ; 若状态位为1，跳转至nx0
	mov	al,buf                ; 否则加载电机相序
	ror	al,1                  ; 进行右移（正转）
	jmp	nx1                   ; 跳转至nx1
nx0:	mov	al,buf              ; 加载电机相序
	rol	al,1                  ; 进行左移（反转）
nx1:	mov	dx,io8255_c           ; 加载8255控制器C的地址到DX
	out	dx,al                 ; 输出新的电机相序
	mov	buf,al                ; 更新电机相序缓冲区

; -----------------------------------交通灯显示--------------------------
light:	
	call	digital              ; 调用数码管显示程序
			
	mov	dx,io8255_b           ; 加载8255控制器B的地址到DX
greeen1:cmp	count1,500          ; 判断是否到5秒，输出绿灯
	ja	yellow1               ; 若大于500，跳转至yellow1
	mov	al,portcl             ; 输出主干道通行信号
	out	dx,al                 ; 输出到8255
	jmp	ok                   ; 跳转至ok
yellow1:cmp	count1,700          ; 判断是否到7秒，切换到黄灯
	ja	greeen2              ; 若大于700，跳转至greeen2
	mov	al,portcl[1]         ; 输出主干道黄灯状态
	out	dx,al                 ; 输出到8255
	jmp	ok                   ; 跳转至ok
greeen2:cmp	count1,1000         ; 判断是否到10秒，输出从干道通行
	ja	yellow2              ; 若大于1000，跳转至yellow2
	mov	al,portcl[2]         ; 输出从干道通行信号
	out	dx,al                 ; 输出到8255
	jmp	ok                   ; 跳转至ok
yellow2:cmp	count1,1200         ; 判断是否到12秒，切换到黄灯
	ja	finish               ; 若大于1200，跳转至finish
	mov	al,portcl[3]         ; 输出从干道黄灯状态
	out	dx,al                 ; 输出到8255
	jmp	ok                   ; 跳转至ok
finish:	mov	count1,0           ; 一次循环结束后，重置计数器
ok:	call	digital              ; 调用数码管显示程序
	jmp	lop1                 ; 返回到主循环

; ---------------------------倒计时数码管显示-----------------------------
digital	proc
begin:
	mov	dx,io8255_a           ; 加载8255控制器A的地址到DX
zhu5:	cmp	count1,100           ; 判断计数器是否大于100
	ja	zhu4                   ; 若大于100，跳转至zhu4
	mov	al,6dh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhu4:	cmp	count1,200           ; 判断计数器是否大于200
	ja	zhu3                   ; 若大于200，跳转至zhu3
	mov	al,66h                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhu3:	cmp	count1,300           ; 判断计数器是否大于300
	ja	zhu2                   ; 若大于300，跳转至zhu2
	mov	al,4fh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhu2:	cmp	count1,400           ; 判断计数器是否大于400
	ja	zhu1                   ; 若大于400，跳转至zhu1
	mov	al,5bh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhu1:	cmp	count1,500           ; 判断计数器是否大于500
	ja	zhu22                  ; 若大于500，跳转至zhu22
	mov	al,06h                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhu22:	cmp	count1,600           ; 判断计数器是否大于600
	ja	zhu11                  ; 若大于600，跳转至zhu11
	mov	al,5bh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhu11:	cmp	count1,700           ; 判断计数器是否大于700
	ja	zhi3                   ; 若大于700，跳转至zhi3
	mov	al,06h                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhi3:	cmp	count1,800           ; 判断计数器是否大于800
	ja	zhi2                   ; 若大于800，跳转至zhi2
	mov	al,4fh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhi2:	cmp	count1,900           ; 判断计数器是否大于900
	ja	zhi1                   ; 若大于900，跳转至zhi1
	mov	al,5bh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhi1:	cmp	count1,1000          ; 判断计数器是否大于1000
	ja	zhi22                  ; 若大于1000，跳转至zhi22
	mov	al,06h                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhi22:	cmp	count1,1100          ; 判断计数器是否大于1100
	ja	zhi11                  ; 若大于1100，跳转至zhi11
	mov	al,5bh                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
	jmp	exit                  ; 跳转至exit
zhi11:	cmp	count1,1200          ; 判断计数器是否大于1200
	ja	exit                   ; 若大于1200，跳转至exit
	mov	al,06h                 ; 设置数码管显示的值
	out	dx,al                 ; 输出到8255
exit:	ret                     ; 返回

digital		endp                 ; 数码管显示程序结束
; ------------------------------延时程序---------------------------------
delay	proc	near
	push	ax                    ; 保存AX寄存器
	push	cx                    ; 保存CX寄存器
	push	dx                    ; 保存DX寄存器
	mov	dl,input              ; 将输入值加载到DL
x1:	mov	cx,0ffh              ; 设置CX为255
x2:	loop	x2                  ; 循环，直到CX为0
	dec	dl                    ; DL减1
	jnz	x1                   ; 若DL不为0，返回x1
	pop	dx                    ; 恢复DX寄存器
	pop	cx                    ; 恢复CX寄存器
	pop	ax                    ; 恢复AX寄存器
	ret                        ; 返回

delay	endp                     ; 延时程序结束
; -----------------------------中断服务程序---------------------------------
int_proc	proc	far
	push	ax                    ; 保存AX寄存器
	push	dx                    ; 保存DX寄存器
int1:	inc	count1              ; 中断计数器加1
	mov	al,20h                 ; 发送EOI到8259
	mov	dx,io8259_0           ; 加载主8259的I/O地址到DX
	out	dx,al                 ; 输出EOI到8259
	pop	dx                    ; 恢复DX寄存器
	pop	ax                    ; 恢复AX寄存器
	sti                        ; 使能中断
	iret                       ; 中断返回

int_proc	endp                   ; 中断服务程序结束
code	ends                     ; 代码段结束
end	start                    ; 程序入口